#!/usr/bin/env python3

import argparse
import juliandate as jd


def convert(j, args):
    d = int(j + 0.5) if args.round else j
    c = jd.to_julian(d) if args.julian else jd.to_gregorian(d)

    if args.tabs:
        return "\t".join([str(x) for x in c])

    return ", ".join([str(x) for x in c])
    

def main():
    parser = argparse.ArgumentParser(description="Convert Julian Days to calendar days")
    parser.add_argument("jds", metavar='J', type=float, nargs='*',
                        help="Julian days to convert")
    parser.add_argument("-f", "--file", type=argparse.FileType('r'),
                        help="Read from file or stdin ('-')")
    parser.add_argument("-j", "--julian", action="store_true",
                        help="Convert to Julian date instead of Gregorian")
    parser.add_argument("-r", "--round", action="store_true",
                        help="Round to simple day")
    parser.add_argument("-t", "--tabs", action="store_true",
                        help="Separate output with tabs instead of commas")

    args = parser.parse_args()

    if args.file is None:
        for j in args.jds:
            print(convert(j, args))

    else:
        for j in args.file:
            try:
                print(convert(float(j), args))
            except ValueError:
                pass


if __name__ == "__main__":
    main()



import math
from juliandate.__version__ import __version__


def to_julian(J):
    """Return a Julian calendar date for a Julian Day."""
    return __jd_to_date(int(J + 0.5) + 1401) + __h_m_s(J)


def to_gregorian(J):
    """Return a Gregorian calendar date for a Julian Day."""
    j = 1401
    B = 274277
    C = -38

    return __jd_to_date(
        J + j + int(((int((4 * J + B) / 146097)) * 3) / 4) + C
    ) + __h_m_s(J)


def from_gregorian(Y, M, D, H=0, m=0, sec=0, ms=0):
    """Return a Julian day for a Gregorian calendar date."""
    return (
        int((1461 * (Y + 4800 + int((M - 14) / 12))) / 4)
        + int((367 * (M - 2 - 12 * int((M - 14) / 12))) / 12)
        - int((3 * int((Y + 4900 + int((M - 14) / 12)) / 100)) / 4)
        + D
        - 32075
    ) + __day_pct(H, m, sec, ms)


def from_julian(Y, M, D, H=0, m=0, sec=0, ms=0):
    """Return a Julian day for a Julian calendar date."""
    return (
        367 * Y
        - int((7 * (Y + 5001 + int((M - 9) / 7))) / 4)
        + int((275 * M) / 9)
        + D
        + 1729777
        + __day_pct(H, m, sec, ms)
    )


def __jd_to_date(f):
    """Base calculation for converting from Julian day.

    see https://en.wikipedia.org/wiki/Julian_day
    """
    m = 2
    n = 12
    p = 1461
    r = 4
    s = 153
    u = 5
    v = 3
    w = 2
    y = 4716

    e = r * f + v
    g = int((e % p) / r)
    h = u * g + w

    D = int((h % s) / u) + 1
    M = ((int(h / s) + m) % n) + 1
    Y = int(e / p) - y + int((n + m - M) / n)

    return (Y, M, D)


def __h_m_s(t):
    """ Convert decimal fraction to hours, minutes, (fractional) seconds. """
    pct = t - int(t)
    (hour, r) = __tdiv(pct, 24)
    (minutes, r) = __tdiv(r, 60)
    (seconds, r) = __tdiv(r, 60)
    return ((hour + 12) % 24, minutes, seconds, math.floor(r * 1_000_000 + 0.5))


def __tdiv(t, d):
    return (int(d * t), d * t - int(d * t))


def __day_pct(h, m, sec, ms):
    s = sec + (ms/1_000_000)
    return ((h * 3600 + m * 60 + s) / 86400) - 0.5


def version():
    return __version__




import juliandate as jd

def test_gregorian():
    # A.D. 1969 July 20 20:17:30.0, some precision lacking
    assert jd.to_gregorian(2440423.345486) == (1969, 7, 20, 20, 17, 29, 990416)


def test_julian():
    # 44 March 15 B.C. 15:00:00.0
    print(jd.to_julian(1705426.125000))
    assert jd.to_julian(1705426.125000) == (-43, 3, 15, 15, 0, 0, 0)


def test_from_gregorian():
    assert jd.from_gregorian(1969, 7, 20) == 2440422.5
    assert jd.from_gregorian(1969, 7, 20, 20, 17, 30) == 2440423.345486111
    assert jd.from_gregorian(1969, 7, 20, 8, 17, 30) == 2440422.845486111


def test_from_julian():
    assert jd.from_julian(-43, 3, 15) == 1705425.5
    assert jd.from_julian(-43, 3, 15, 15) == 1705426.125
    assert jd.from_julian(-43, 3, 15, 3) == 1705425.625


def test_rollover():
    # Noon, 0.0 should be hour 12
    assert jd.to_julian(1705426) == (-43, 3, 15, 12, 0, 0, 0)

    # 0.25 should be hour 18
    assert jd.to_julian(1705426.25) == (-43, 3, 15, 18, 0, 0, 0)
    
    # Midnight, 0.5, should be zero
    assert jd.to_julian(1705426.5) == (-43, 3, 16, 0, 0, 0, 0)

    # 0.75 should be 6 AM
    assert jd.to_julian(1705426.75) == (-43, 3, 16, 6, 0, 0, 0)


def test_rounding():
    assert jd.to_julian(1566223.56309468) == (-424, 2, 2, 1, 30, 51, 380349)


def test_version():
    assert jd.version() == "1.0.3"


def test_julian_day0():
    assert jd.to_julian(0) == (-4712, 1, 1, 12, 0, 0, 0)
    assert jd.from_julian(-4712, 1, 1, 12, 0, 0, 0) == 0


def test_gregorian_day0():
    assert jd.to_gregorian(0) == (-4713, 11, 24, 12, 0, 0, 0)
    assert jd.from_gregorian(-4713, 11, 24, 12, 0, 0, 0) == 0


def test_for_rounding_error():
    # Some values that were subject to from_gregorian rounding error
    # (Issue #2)
    assert jd.from_gregorian(*jd.to_gregorian(187100)) == 187100
    assert jd.from_gregorian(*jd.to_gregorian(150576)) == 150576
    assert jd.from_gregorian(*jd.to_gregorian(41003)) == 41003
    assert jd.from_gregorian(*jd.to_gregorian(28951)) == 28951
    assert jd.from_gregorian(*jd.to_gregorian(4479)) == 4479
    assert jd.from_gregorian(*jd.to_gregorian(0)) == 0


def test_fliegel_van_flandern():
    # Fliegel, Henry F., and Thomas C. Van Flandern. “A Machine
    # Algorithm for Processing Calendar Dates.” Communcations of the
    # ACM 11, no. 10 (1968): 657.
    assert jd.from_gregorian(1970, 1, 1, 12) == 2440588
